
# q-expansion of newform 162.7.d.d, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 162
weight = 7
poly_data = [4, 0, -2, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[0, 0, 1, 0], 2], [[0, 0, 0, 1], 2]]

hecke_ring_character_values = [[83, [1, 0, -1, 0]]]
aps_data = [[0, -4, 0, 4], [0, 0, 0, 0], [0, 123, 0, 0], [0, 0, 484, 0], [0, -948, 0, 948], [-3368, 0, 3368, 0], [0, 0, 0, -9], [5744, 0, 0, 0], [0, -2388, 0, 0], [0, -20757, 0, 20757], [39796, 0, -39796, 0], [52526, 0, 0, 0], [0, -26193, 0, 0], [0, 0, -3800, 0], [0, -54300, 0, 54300], [0, 0, 0, -168813], [0, -176664, 0, 0], [0, 0, -13250, 0], [-168968, 0, 168968, 0], [0, 0, 0, 375804], [236144, 0, 0, 0], [0, 0, 35116, 0], [0, 7764, 0, -7764], [0, 0, 0, 91449], [0, 0, 321424, 0], [0, -472899, 0, 472899], [-1993412, 0, 1993412, 0], [0, 0, 0, 184320], [194456, 0, 0, 0], [0, -581169, 0, 0], [3057212, 0, 0, 0], [0, 2173560, 0, 0], [0, 3170751, 0, -3170751], [1092328, 0, -1092328, 0], [0, -1570395, 0, 0], [0, 0, 4078708, 0], [-6155678, 0, 6155678, 0], [800696, 0, 0, 0], [0, 3397416, 0, 0], [0, 2524317, 0, -2524317], [0, 0, 0, 5258736], [-10381192, 0, 0, 0], [0, -9188184, 0, 9188184], [3931954, 0, -3931954, 0], [0, 0, 0, -3804003], [-565900, 0, 0, 0], [13516504, 0, -13516504, 0], [0, 0, 5354836, 0], [0, 9621132, 0, -9621132], [-4346408, 0, 4346408, 0], [0, 0, 0, 14307111], [0, -14421216, 0, 0], [0, 0, 3120928, 0], [0, 0, 0, -3602844], [0, 10208769, 0, 0], [0, -22101816, 0, 22101816], [0, 0, 0, -177723], [29639900, 0, 0, 0], [0, 0, -13221272, 0], [0, 4330041, 0, -4330041], [6743248, 0, -6743248, 0], [0, -7087971, 0, 0], [-5230600, 0, 0, 0], [0, 22077384, 0, 0], [0, 0, -22477826, 0], [0, 19531077, 0, -19531077], [0, 0, 5761384, 0], [40105168, 0, -40105168, 0], [0, 47950836, 0, 0], [0, 0, 42063838, 0], [0, 12443361, 0, -12443361], [0, 0, 0, 9893844], [0, 0, 26585476, 0], [-17882930, 0, 17882930, 0], [72097760, 0, 0, 0], [0, 6142272, 0, 0], [0, 34960155, 0, -34960155], [15691106, 0, 0, 0], [0, -33553191, 0, 0], [115511632, 0, -115511632, 0], [0, 103727724, 0, 0], [0, 0, -139238840, 0], [0, 0, 0, 70987572], [-40063138, 0, 0, 0], [0, 0, 138591844, 0], [0, 78801828, 0, -78801828], [0, 0, 0, 43215993], [0, 0, -35666480, 0], [0, -107468013, 0, 107468013], [-114978116, 0, 114978116, 0], [0, 0, 0, -62345988], [0, 63242772, 0, -63242772], [-75168844, 0, 0, 0], [0, 31877952, 0, 0], [-91545824, 0, 91545824, 0], [0, 0, 0, 113874372], [0, 16970181, 0, 0], [0, 0, 0, 63703071], [-37769056, 0, 0, 0], [25479992, 0, 0, 0], [0, 0, -205215896, 0], [0, 0, 0, 170513523], [0, -119414172, 0, 0], [0, 172530591, 0, -172530591], [-241501808, 0, 241501808, 0], [-49397902, 0, 0, 0], [0, 121659144, 0, -121659144], [0, 0, 0, -191373777], [0, -122541204, 0, 0], [0, 0, 431090242, 0], [-16699052, 0, 16699052, 0], [-192320782, 0, 0, 0], [0, -132245481, 0, 0], [0, 0, -254873216, 0], [92360180, 0, 0, 0], [0, 300283863, 0, -300283863], [-375946328, 0, 375946328, 0], [0, 0, 0, -18649764], [0, 182968467, 0, 0], [0, 98531736, 0, -98531736], [472544818, 0, -472544818, 0], [0, 0, -548833250, 0], [0, 71248245, 0, -71248245], [0, 0, 0, 221364], [0, 0, 372812200, 0], [0, 0, 0, -439233525], [0, 0, 246510232, 0], [0, 0, 0, 67909788], [0, 0, -391370852, 0], [-34907816, 0, 34907816, 0], [-302999344, 0, 0, 0], [0, 173685480, 0, 0], [82327012, 0, -82327012, 0], [-603578680, 0, 0, 0], [0, -164743329, 0, 0], [-815795570, 0, 815795570, 0], [0, 0, 0, 259215939], [402462160, 0, -402462160, 0], [0, -366945645, 0, 0], [0, 0, 0, 2149857], [225521120, 0, 0, 0], [0, -195893571, 0, 195893571], [707335828, 0, -707335828, 0], [0, 0, 0, 188334792], [503826392, 0, 0, 0], [0, -539780124, 0, 539780124], [0, 0, 18798514, 0], [0, 485377377, 0, -485377377], [-551931608, 0, 551931608, 0], [0, 0, 0, -258564168], [585387070, 0, -585387070, 0], [0, 0, 0, 303886863], [220084904, 0, 0, 0], [0, 828702744, 0, 0], [0, 0, -731614232, 0], [0, -649544808, 0, 649544808], [-215986660, 0, 0, 0], [0, -219290889, 0, 219290889], [-742448338, 0, 0, 0], [0, 128527629, 0, 0], [0, -607538568, 0, 607538568], [0, 0, 0, -485654193], [1094109820, 0, -1094109820, 0], [0, 0, 0, -313329276], [0, 841488423, 0, 0], [0, 835705176, 0, -835705176], [509602268, 0, 0, 0], [0, 0, 990780046, 0]]
